from django_rpyc.decorators import register
from django_rpyc.server import DjangoRpycService


def test_register_with_name():
    @register("my_service")
    class MyService:
        pass

    assert "my_service" in DjangoRpycService.MAPPING
    assert DjangoRpycService.MAPPING["my_service"] == MyService


def test_register_without_name():
    @register
    class MyService:
        pass

    assert "myservice" in DjangoRpycService.MAPPING
    assert DjangoRpycService.MAPPING["myservice"] == MyService


def test_register_with_non_string_name():
    @register(123)
    class MyService:
        pass

    assert "myservice" in DjangoRpycService.MAPPING
    assert DjangoRpycService.MAPPING[123] == MyService


def test_register_with_name_printing(capfd):
    @register("my_service")
    class MyService:
        pass

    captured = capfd.readouterr()
    assert "Registered: Name: my_service, Class: MyService" in captured.out


def test_register_without_name_printing(capfd):
    @register
    class MyService:
        pass

    captured = capfd.readouterr()
    assert "Registered: Name: myservice, Class: MyService" in captured.out


def test_register_with_non_string_name_printing(capfd):
    @register(123)
    class MyService:
        pass

    captured = capfd.readouterr()
    assert "Registered: Name: 123, Class: MyService" in captured.out



import logging
from typing import Type, Callable, Optional, Any

from .server import DjangoRpycService


def register(name: Optional[Any] = None) -> Callable[[Type], Type]:
    """
    Decorator function used to register a service in Django Rpyc.

    Args:
        name (Optional[Any]): The name of the service. If not provided, the name will be derived from the class name.

    Returns:
        Callable[[Type], Type]: The decorator function.

    Example:
    ```
        @register("my_service")
        class MyService:
            pass
    ```
        This will register the service with the name `my_service`.

    ```
        @register
        class MyService:
            pass
    ```

        This will register the service with the name `myservice`.
    """

    logging.info(f"Registering service {name}")

    def decorator(service: Type) -> Type:
        nonlocal name

        if callable(name):
            name = service.__name__.lower()

        logging.info(f"Registered: Name: {name}, Class: {service.__name__}")

        DjangoRpycService.add_service(name, service)
        print(f"Registered: Name: {name}, Class: {service.__name__}")

        return service

    return decorator if not callable(name) else decorator(name)



from django_rpyc.server import DjangoRpycService


def test_add_service():

    class MyService:
        pass

    name = "my_service"
    service = MyService

    result = DjangoRpycService.add_service(name, service)

    assert result == service
    assert name in DjangoRpycService.MAPPING
    assert DjangoRpycService.MAPPING[name] == service


def test_get_service_names():

    class MyService:
        pass

    DjangoRpycService.MAPPING.clear()

    name = "my_service"
    service = MyService

    DjangoRpycService.add_service(name, service)

    expected_names = [name]
    obj = DjangoRpycService()
    actual_names = obj.exposed_get_service_names()

    assert actual_names == expected_names


def test_exposed_use_existing_service():
    class MyService:
        pass

    DjangoRpycService.MAPPING.clear()

    name = "my_service"
    service = MyService

    DjangoRpycService.add_service(name, service)

    obj = DjangoRpycService()
    result = obj.exposed_use(name)

    assert isinstance(result, MyService)


def test_exposed_use_nonexistent_service():
    DjangoRpycService.MAPPING.clear()

    name = "nonexistent_service"

    obj = DjangoRpycService()

    try:
        obj.exposed_use(name)
        assert False, "Expected ValueError to be raised"
    except ValueError as e:
        assert str(e) == f"Service {name} not found"



import os
import importlib.util

from django.apps import AppConfig, apps
from django.conf import settings


class DjangoRpycConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_rpyc"

    def ready(self) -> None:
        """
        Executes code when the Django application is ready.

        This method is called when the Django application is fully loaded and ready to run.
        It loads the specified module for each app in the project, if the module exists.

        Reason being that the module may contain code that needs to be executed when the Django
        application is ready so that it does not require manual intervention by importing
        the module in the project's `__init__.py` file or corresponding app's `apps.py` ready method.

        Returns:
            None
        """
        DJANGO_RPYC_FILENAME = getattr(settings, "DJANGO_RPYC_FILENAME", "django_rpyc")

        for app in apps.get_app_configs():
            try:
                module_path = os.path.join(app.path, f"{DJANGO_RPYC_FILENAME}.py")

                if os.path.exists(module_path):
                    spec = importlib.util.spec_from_file_location(
                        f"{app.name}.{DJANGO_RPYC_FILENAME}", module_path
                    )
                    module = importlib.util.module_from_spec(spec)
                    spec.loader.exec_module(module)

            except Exception as e:
                print(f"Failed to load {DJANGO_RPYC_FILENAME} from {app.name}: {e}")

        return super().ready()



from typing import Dict, Type

import rpyc
from rpyc.utils.server import ThreadedServer


class DjangoRpycService(rpyc.Service):
    """
    A class that represents a Django Rpyc Service.

    This class extends the `rpyc.Service` class and provides methods for adding services,
    getting service names, and using a specific service.

    Attributes:
        MAPPING (Dict[str, Type[rpyc.Service]]): A dictionary that maps service names to their corresponding service classes.
    """

    MAPPING: Dict[str, Type[rpyc.Service]] = {}

    @classmethod
    def add_service(cls, name: str, service: Type[rpyc.Service]):
        """
        Adds a service to the mapping.

        Args:
            name (str): The name of the service.
            service (Type[rpyc.Service]): The service class.

        Returns:
            Type[rpyc.Service]: The added service class.
        """
        cls.MAPPING[name] = service
        return service

    def exposed_get_service_names(self):
        """
        Returns a list of service names.

        Returns:
            List[str]: A list of service names.
        """
        return list(self.MAPPING.keys())

    def exposed_use(self, name: str):
        """
        Uses a specific service.

        Args:
            name (str): The name of the service to use.

        Returns:
            rpyc.Service: An instance of the specified service.

        Raises:
            ValueError: If the specified service is not found in the mapping.
        """
        if name not in self.MAPPING:
            raise ValueError(f"Service {name} not found")
        return self.MAPPING[name]()



from typing import Union
from setuptools import setup, find_packages


def get_version() -> Union[str, None]:
    with open("django_rpyc/__init__.py") as f:
        for line in f:
            if line.startswith("__version__"):
                return line.split("=")[1].strip().strip("'")
setup(
    version=get_version(),
)


import logging
from rpyc.utils.server import ThreadedServer

from django.core.management.base import BaseCommand
from django.conf import settings


class Command(BaseCommand):
    help = "Starts the RPyC server"

    def add_arguments(self, parser):
        parser.add_argument(
            "--port",
            type=int,
            default=getattr(settings, "DJANGO_RPYC_PORT", 18861),
            help="The port to start the RPyC server on",
        )

    def handle(self, *args, **options):
        from django_rpyc.server import DjangoRpycService

        logging.info("Starting RPyC server...")
        t = ThreadedServer(DjangoRpycService, port=options["port"])
        t.start()
